/**
 * $Id: IOExceptionAspect.aj,v 1.5 2008/09/16 20:46:16 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 10.09.2008 by oliver (ob@oasd.de)
 */
package patterntesting.io;

import java.io.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

/**
 * Often the thrown IOExceptions contains too less informations so you don't
 * know what happens. So this aspect try to catch all methods from the
 * java.io package which may throw an IOException. It tries to enrich the
 * exception with additional informations like the filename wich was not found
 * or other useful infos.
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 10.09.2008
 * @version $Revision: 1.5 $
 * 
 * TODO provide an annotation 'NoBetterIOExceptions'
 */
public aspect IOExceptionAspect {
	
	private static Log log = LogFactoryImpl.getLog(IOExceptionAspect.class);

	/**
	 * If File.createTempFile(String prefix, String suffix, File directory)
	 * throws an IOException it may be that the directory does not exist in
	 * the temp directory. If it is so the exception should contain the missing
	 * directory name.
	 * 
	 * @param dir
	 * @return
	 * @throws IOException
	 * @see {@link File#createTempFile(String, String, File)}
	 */
	File around(File dir) throws IOException :  
			call(public static File File.createTempFile(String, String, File) throws IOException) 
			&& args(*, *, dir) {
		try {
			return proceed(dir);
		} catch (IOException ioe) {
			File tmpdir = FileHelper.getTmpdir(dir);
			if (tmpdir.exists()) {
				throw ioe;
			} else {
				throw betterIOException(ioe, tmpdir);
			}
		}
	}
	
	/**
	 * If File.createTempFile(String prefix, String suffix)
	 * throws an IOException it may be that the prefix may contain a directory
	 * part which does not exist. Or perhaps the suffix is wrong.
	 * 
	 * @param dir
	 * @return
	 * @throws IOException
	 * @see {@link File#createTempFile(String, String)}
	 */
	File around(String prefix, String suffix) throws IOException :  
			call(public static File File.createTempFile(String, String) throws IOException) 
			&& args(prefix, suffix) {
		try {
			return proceed(prefix, suffix);
		} catch (IOException ioe) {
			String dirname = new File(prefix).getParent();
			if (dirname != null) {
				File tmpdir = FileHelper.getTmpdir(dirname);
				if (!tmpdir.exists()) {
					throw betterIOException(ioe, tmpdir);
				}
			}
			dirname = new File(suffix).getParent();
			if (dirname != null) {
				String msg = "invalid suffix '" + suffix + "'";
				log.debug("converting: " + ioe.getMessage() + " -> " + msg);
				throw new IOException(msg);
			}
			throw ioe;
		}
	}
	
	/**
	 * We want to see the file as additional information if an IOException
	 * is thrown.
	 */
	Object around(File file) throws IOException :  
			call(public !static * File.*(..) throws IOException) 
			&& target(file) {
		try {
			return proceed(file);
		} catch (IOException ioe) {
			throw betterIOException(ioe, file);
		}
	}
	
	
	private static IOException betterIOException(IOException ioe, File file) {
		return betterIOException(ioe, file.getAbsolutePath());
	}
	
	private static IOException betterIOException(IOException ioe,
			String filename) {
		return FileHelper.betterIOException(ioe, filename);
	}

}

/**
 * $Log: IOExceptionAspect.aj,v $
 * Revision 1.5  2008/09/16 20:46:16  oboehm
 * switch to commons-logging for more flexibility
 *
 * Revision 1.4  2008/09/16 20:18:15  oboehm
 * concrete StackTraceAspect and SystemAspect added
 * (can be disabled via annotations)
 *
 * Revision 1.3  2008/09/11 08:13:57  oboehm
 * IOException from File methods contains now the fail
 * in the exception message
 *
 * Revision 1.2  2008/09/10 20:53:41  oboehm
 * javadoc comment added
 *
 * Revision 1.1  2008/09/10 20:34:12  oboehm
 * begin with IOExceptionAspect for IOExceptions with better error messages
 *
 * $Source: /cvsroot/patterntesting/PatternTesting05/src/main/aspect/patterntesting/io/IOExceptionAspect.aj,v $
 */
