/**
 * $Id: SystemAspect.aj,v 1.1 2008/09/16 20:18:15 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 14.09.2008 by oliver (ob@oasd.de)
 */
package patterntesting.java;

import java.io.PrintStream;
import patterntesting.java.annotation.*;

/**
 * Pattern Test that ensures that <code>System.out</code> and
 * <code>System.err</code> are not used in the code. This is
 * to prevent the usage of <code>System.out.println("")</code>
 * calls (and variations).
 * 
 * Note: With AspectJ it is not possible to write a compile time
 * check that only ensures that <code>println()</code> calls are
 * not allowed. It is possible for runtime checks but we have
 * preferred to make a compile-time test for simplicity. Thus,
 * code that manipulate <code>System.out</code> and 
 * <code>System.err</code> for other purpose that using the
 * <code>println()</code> methods will need to be added to the
 * list of items to exclude from the tests, hence the abstract
 * <code>allowedCode</code> pointcut.
 *
 * The difference to AbstractSopTest is that is not abstract - the check
 * is on by default and you have not to define the abstract pointcuts
 * "applicationCode" or "allowedCode". And it is only a warning when
 * the user violates the check which can be disabled by the annotations
 * "@SystemOutNeeded" and "@SystemErrNeeded".
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 14.09.2008
 * @version $Revision: 1.1 $
 * 
 * @see AbstractSopTest
 */
public aspect SystemAspect {

    /**
     * Declare logging to stdout as warning
     */
    declare warning: get(PrintStream System.out)
    		&& !within(@SystemOutNeeded *)
    		&& !withincode(@SystemOutNeeded *..*.new(..))
    		&& !withincode(@SystemOutNeeded * *..*.*(..)) : 
        "No logging should be done using System.out!";

    /**
     * Declare logging to stderr as warning
     */
    declare warning: get(PrintStream System.err)
    		&& !within(@SystemErrNeeded *)
    		&& !withincode(@SystemErrNeeded *..*.new(..))
    		&& !withincode(@SystemErrNeeded * *..*.*(..)) : 
        "No logging should be done using System.err!";

}

/**
 * $Log: SystemAspect.aj,v $
 * Revision 1.1  2008/09/16 20:18:15  oboehm
 * concrete StackTraceAspect and SystemAspect added
 * (can be disabled via annotations)
 *
 * $Source: /cvsroot/patterntesting/PatternTesting05/src/main/aspect/patterntesting/java/SystemAspect.aj,v $
 */
