/**
 * $Id: ThreadSafeAspect.aj,v 1.1 2008/10/02 20:36:32 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 30.09.2008 by oliver (ob@oasd.de)
 */
package patterntesting.java.runtime;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

import patterntesting.java.runtime.annotation.*;

/**
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 30.09.2008
 * @version $Revision: 1.1 $
 */
public aspect ThreadSafeAspect {
	
	private static Log log = LogFactoryImpl.getLog(ThreadSafeAspect.class);
	
	/**
	 * Address all methods which are not synchronized.
	 * Constructors can't be synchronized (really?) so they are not
	 * addressed here.
	 */
	pointcut unsynchronizedMethods(Object x) :
		execution(!synchronized !static * *..*.*(..))
		&& this(x)
		&& within(@ThreadSafe *)
		;
	
	pointcut unsynchronizedStaticMethods() :
		execution(!synchronized static * *..*.*(..))
		&& within(@ThreadSafe *)
		;
	
	/**
	 * I don't know if you need the constructor synchronized.
	 * But if you really need it mark it as "@ThreadSafe".
	 */
	pointcut threadsafeConstructors() :
		execution(@ThreadSafe *..*.new(..));
	
	/**
	 * Synchronizes the execution of a method or constructor.
	 * @return the unchanged result of the executed method.
	 */
	Object around() : 
			(unsynchronizedStaticMethods() || threadsafeConstructors())
			&& !@annotation(ThreadSafe) {
		if (log.isTraceEnabled()) {
			log.trace("synchronizing " + thisJoinPoint + "...");
		}
		synchronized (thisEnclosingJoinPointStaticPart) {
			return proceed();
		}
	}

	/**
	 * Synchronizes the execution of a method or constructor.
	 * @return the unchanged result of the executed method.
	 */
	Object around(ThreadSafe t) : 
			unsynchronizedStaticMethods() && @annotation(t) {
		if (t.value().equals(Synchronization.NOT_NEEDED)) {
			if (log.isTraceEnabled()) {
				log.trace("no synchronization needed for " + thisJoinPoint + "...");
			}
			return proceed(t);			
		} else {
			if (log.isTraceEnabled()) {
				log.trace("synchronizing " + thisJoinPoint + "...");
			}
			synchronized (thisEnclosingJoinPointStaticPart) {
				return proceed(t);
			}
		}
	}

	/**
	 * Synchronizes the execution of a method.
	 * @return the unchanged result of the executed method.
	 */
	Object around(Object x) : 
			unsynchronizedMethods(x) && !@annotation(ThreadSafe) {
		if (log.isTraceEnabled()) {
			log.trace("synchronizing " + thisJoinPoint + "...");
		}
		synchronized (x) {
			return proceed(x);
		}
	}
	
	/**
	 * Synchronizes the execution of a method.
	 * @return the unchanged result of the executed method.
	 */
	Object around(Object x, ThreadSafe t) : 
			unsynchronizedMethods(x) && @annotation(t) {
		if (t.value().equals(Synchronization.NOT_NEEDED)) {
			if (log.isTraceEnabled()) {
				log.trace("no synchronization needed for " + thisJoinPoint + "...");
			}
			return proceed(x, t);			
		} else {
			if (log.isTraceEnabled()) {
				log.trace("synchronizing " + thisJoinPoint + "...");
			}
			synchronized (x) {
				return proceed(x, t);
			}
		}
	}

}

/**
 * $Log: ThreadSafeAspect.aj,v $
 * Revision 1.1  2008/10/02 20:36:32  oboehm
 * methods in @TreadSafe classes will be synchronized
 *
 * $Source: /cvsroot/patterntesting/PatternTesting05/src/main/aspect/patterntesting/java/runtime/ThreadSafeAspect.aj,v $
 */
